@@ -164,16 +164,6 @@ then
     chmod 640 $1/drivers/human/tracks/e-track-3/car-torcs.xml
 fi
 mkdir -p $1/drivers/human/tracks/e-track-4 2>/dev/null
-if [ ! -e  $1/drivers/human/tracks/e-track-4/car-360-modena.xml ] || [ drivers/human/tracks/e-track-4/car-360-modena.xml -nt $1/drivers/human/tracks/e-track-4/car-360-modena.xml ]
-then
-    if [ -e $1/drivers/human/tracks/e-track-4/car-360-modena.xml ]
-    then
-        echo "Saving $1/drivers/human/tracks/e-track-4/car-360-modena.xml to $1/drivers/human/tracks/e-track-4/car-360-modena.xml.old"
-        cp -f $1/drivers/human/tracks/e-track-4/car-360-modena.xml $1/drivers/human/tracks/e-track-4/car-360-modena.xml.old
-    fi
-    cp -f drivers/human/tracks/e-track-4/car-360-modena.xml $1/drivers/human/tracks/e-track-4/car-360-modena.xml
-    chmod 640 $1/drivers/human/tracks/e-track-4/car-360-modena.xml
-fi
 if [ ! -e  $1/drivers/human/tracks/e-track-4/car-mclaren-f1.xml ] || [ drivers/human/tracks/e-track-4/car-mclaren-f1.xml -nt $1/drivers/human/tracks/e-track-4/car-mclaren-f1.xml ]
 then
     if [ -e $1/drivers/human/tracks/e-track-4/car-mclaren-f1.xml ]
@@ -267,16 +257,6 @@ then
     chmod 640 $1/drivers/human/tracks/e-track-6/car-torcs.xml
 fi
 mkdir -p $1/drivers/human/tracks/g-track-2 2>/dev/null
-if [ ! -e  $1/drivers/human/tracks/g-track-2/car-360-modena.xml ] || [ drivers/human/tracks/g-track-2/car-360-modena.xml -nt $1/drivers/human/tracks/g-track-2/car-360-modena.xml ]
-then
-    if [ -e $1/drivers/human/tracks/g-track-2/car-360-modena.xml ]
-    then
-        echo "Saving $1/drivers/human/tracks/g-track-2/car-360-modena.xml to $1/drivers/human/tracks/g-track-2/car-360-modena.xml.old"
-        cp -f $1/drivers/human/tracks/g-track-2/car-360-modena.xml $1/drivers/human/tracks/g-track-2/car-360-modena.xml.old
-    fi
-    cp -f drivers/human/tracks/g-track-2/car-360-modena.xml $1/drivers/human/tracks/g-track-2/car-360-modena.xml
-    chmod 640 $1/drivers/human/tracks/g-track-2/car-360-modena.xml
-fi
 if [ ! -e  $1/drivers/human/tracks/g-track-2/car-mclaren-f1.xml ] || [ drivers/human/tracks/g-track-2/car-mclaren-f1.xml -nt $1/drivers/human/tracks/g-track-2/car-mclaren-f1.xml ]
 then
     if [ -e $1/drivers/human/tracks/g-track-2/car-mclaren-f1.xml ]
@@ -614,6 +594,27 @@ then
     chmod 640 $1/drivers/human/preferences.xml
 fi
 mkdir -p $1/config 2>/dev/null
+if [ ! -e  $1/config/raceengine.xml ] || [ config/raceengine.xml -nt $1/config/raceengine.xml ]
+then
+    if [ -e $1/config/raceengine.xml ]
+    then
+        echo "Saving $1/config/raceengine.xml to $1/config/raceengine.xml.old"
+        cp -f $1/config/raceengine.xml $1/config/raceengine.xml.old
+    fi
+    cp -f config/raceengine.xml $1/config/raceengine.xml
+    chmod 640 $1/config/raceengine.xml
+fi
+if [ ! -e  $1/config/style.xsl ] || [ config/style.xsl -nt $1/config/style.xsl ]
+then
+    if [ -e $1/config/style.xsl ]
+    then
+        echo "Saving $1/config/style.xsl to $1/config/style.xsl.old"
+        cp -f $1/config/style.xsl $1/config/style.xsl.old
+    fi
+    cp -f config/style.xsl $1/config/style.xsl
+    chmod 640 $1/config/style.xsl
+fi
+mkdir -p $1/config 2>/dev/null
 if [ ! -e  $1/config/screen.xml ] || [ config/screen.xml -nt $1/config/screen.xml ]
 then
     if [ -e $1/config/screen.xml ]
