@@ -4,7 +4,7 @@
     created              : Sat Mar 18 23:16:38 CET 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: human.cpp,v 1.43 2005/08/17 14:06:55 berniw Exp $
+    version              : $Id: human.cpp,v 1.45 2006/09/06 23:46:45 berniw Exp $
 
  ***************************************************************************/
 
@@ -20,7 +20,7 @@
 /** @file   
     		
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: human.cpp,v 1.43 2005/08/17 14:06:55 berniw Exp $
+    @version	$Id: human.cpp,v 1.45 2006/09/06 23:46:45 berniw Exp $
 */
 
 
@@ -87,6 +87,8 @@ static tKeyInfo skeyInfo[256];
 static int currentKey[256];
 static int currentSKey[256];
 
+static double lastKeyUpdate = -10.0;
+
 static int	firstTime = 0;
 
 #ifdef _WIN32
@@ -489,7 +491,7 @@ static void common_drive(int index, tCarElt* car, tSituation *s)
 		car->_raceCmd = RM_CMD_PIT_ASKED;
 	}
 
-	if (index == masterPlayer) {
+	if (lastKeyUpdate != s->currentTime) {
 		/* Update the controls only once for all the players */
 		updateKeys();
 
@@ -498,6 +500,7 @@ static void common_drive(int index, tCarElt* car, tSituation *s)
 		}
 
 		GfctrlMouseGetCurrent(mouseInfo);
+		lastKeyUpdate = s->currentTime;
 	}
 
 	if (((cmd[CMD_ABS].type == GFCTRL_TYPE_JOY_BUT) && joyInfo->edgeup[cmd[CMD_ABS].val]) ||
@@ -1060,6 +1063,27 @@ static int pitcmd(int index, tCarElt* car, tSituation *s)
 
 	car->_pitRepair = (int)car->_dammage;
 
+	int i;
+	int key;
+	tControlCmd *cmd;
+
+	if (HCtx[idx]) {
+		cmd = HCtx[idx]->CmdControl;
+		for (i = 0; i < nbCmdControl; i++) {
+			if (cmd[i].type == GFCTRL_TYPE_KEYBOARD || cmd[i].type == GFCTRL_TYPE_SKEYBOARD) {
+				key = cmd[i].val;
+				keyInfo[key].state = GFUI_KEY_UP;
+				keyInfo[key].edgeDn = 0;
+				keyInfo[key].edgeUp = 0;
+				skeyInfo[key].state = GFUI_KEY_UP;
+				skeyInfo[key].edgeDn = 0;
+				skeyInfo[key].edgeUp = 0;
+				currentKey[key] = GFUI_KEY_UP;
+				currentSKey[key] = GFUI_KEY_UP;
+			}
+		}
+	}
+
 	return ROB_PIT_MENU; /* The player is able to modify the value by menu */
 }
 
