@@ -4,7 +4,7 @@
     created              : Sun Jan 30 12:00:15 CET 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: car.h,v 1.35 2005/07/19 19:55:52 berniw Exp $
+    version              : $Id: car.h,v 1.37 2006/10/12 22:19:22 berniw Exp $
 
  ***************************************************************************/
 
@@ -20,7 +20,7 @@
 /** @file
     		This is the car structure.
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: car.h,v 1.35 2005/07/19 19:55:52 berniw Exp $
+    @version	$Id: car.h,v 1.37 2006/10/12 22:19:22 berniw Exp $
     @ingroup	carstruct
     @note	Short cuts are to be used with the carElt structure.
 */
@@ -73,6 +73,7 @@ typedef struct {
 /** Static Public info */
 typedef struct {
     char	name[MAX_NAME_LEN];	/**< Driver's name */
+	char	teamname[MAX_NAME_LEN];	/**< Team name */
     char	carName[MAX_NAME_LEN];	/**< Car object name */
     char	category[MAX_NAME_LEN];	/**< Car's category */
     int		raceNumber;		/**< Car's race number */
@@ -91,8 +92,9 @@ typedef struct {
 } tInitCar;
 /* structure access short cuts */
 #define _name		info.name			/**< short cut to tInitCar#name */
+#define _teamname	info.teamname		/**< short cut to tInitCar#teamname */
 #define _carName	info.carName			/**< short cut to tInitCar#carName */
-#define _category	info.cartegory			/**< short cut to tInitCar#cartegory */
+#define _category	info.category			/**< short cut to tInitCar#category */
 #define _driverType	info.driverType			/**< short cut to tInitCar#driverType */
 #define _skillLevel	info.skillLevel			/**< short cut to tInitCar#skillLevel */
 #define _raceNumber	info.raceNumber			/**< short cut to tInitCar#raceNumber */
