@@ -1,5 +1,5 @@
 /* -*- Mode: C++; -*- */
-/* VER: $Id: ANN.cpp,v 1.3 2005/08/05 09:02:41 berniw Exp $ */
+/* VER: $Id: ANN.cpp,v 1.4 2005/09/19 12:48:03 berniw Exp $ */
 // copyright (c) 2004 by Christos Dimitrakakis <dimitrak@idiap.ch>
 /***************************************************************************
  *                                                                         *
@@ -1169,7 +1169,7 @@ static inline void WriteToken (char* tag, FILE* f)
 /// Load an ANN from a filename.
 ANN* LoadANN(char* filename)
 {
-	FILE* f = fopen (filename, "r");
+	FILE* f = fopen (filename, "rb");
 	if (f) {
 		ANN* ann = LoadANN (f);
 		fclose (f);
@@ -1180,7 +1180,7 @@ ANN* LoadANN(char* filename)
 /// Save the ANN to a filename.
 int SaveANN(ANN* ann, char* filename)
 {
-	FILE* f = fopen (filename, "w");
+	FILE* f = fopen (filename, "wb");
 	if (f) {
 		int r = SaveANN (ann, f);
 		fclose (f);
