@@ -4,7 +4,7 @@
     created              : Fri Aug 13 22:32:14 CEST 1999
     copyright            : (C) 1999 by Eric Espie                         
     email                : torcs@free.fr   
-    version              : $Id: tgf.h,v 1.40 2005/02/01 15:55:54 berniw Exp $                                  
+    version              : $Id: tgf.h,v 1.41 2005/08/05 09:22:00 berniw Exp $                                  
  ***************************************************************************/
 
 /***************************************************************************
@@ -19,7 +19,7 @@
 /** @file   
     	The Gaming Framework API.
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: tgf.h,v 1.40 2005/02/01 15:55:54 berniw Exp $
+    @version	$Id: tgf.h,v 1.41 2005/08/05 09:22:00 berniw Exp $
 */
 
 
@@ -404,6 +404,7 @@ extern void SetDataDir(char *buf);
 extern int GetSingleTextureMode (void);
 extern void SetSingleTextureMode (void);
 extern int GfNearestPow2 (int x);
+extern int GfCreateDir(char *path);
 
 /*
  * Copyright (c) 1991, 1993
@@ -619,6 +620,9 @@ void GfHashRelease(void *hash, tfHashFree hashFree);
 void *GfHashGetFirst(void *hash);
 void *GfHashGetNext(void *hash);
 
+#define GF_DIR_CREATION_FAILED 0
+#define GF_DIR_CREATED 1
+
 #endif /* __TGF__H__ */
 
 
