@@ -4,7 +4,7 @@
     created              : Fri Aug 13 22:29:56 CEST 1999
     copyright            : (C) 1999, 2004 by Eric Espie, Bernhard Wymann
     email                : torcs@free.fr
-    version              : $Id: screen.cpp,v 1.19 2005/08/05 09:26:39 berniw Exp $
+    version              : $Id: screen.cpp,v 1.21 2006/10/29 17:00:53 berniw Exp $
 ***************************************************************************/
 
 /***************************************************************************
@@ -19,7 +19,7 @@
 /** @file
     Screen management.
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: screen.cpp,v 1.19 2005/08/05 09:26:39 berniw Exp $
+    @version	$Id: screen.cpp,v 1.21 2006/10/29 17:00:53 berniw Exp $
     @ingroup	screen
 */
 
@@ -73,7 +73,7 @@ static int usedFG = 0;
 static char	**Res = NULL;
 static int nbRes = 0;
 #else // USE_RANDR_EXT
-static char	*Res[] = {"640x480", "800x600", "1024x768", "1152x768", "1152x864", "1200x854", "1200x960", "1280x1024", "1600x1200", "320x200"};
+static char	*Res[] = {"640x480", "800x600", "1024x768", "1152x768", "1152x864", "1200x854", "1200x960", "1280x1024", "1400x900", "1600x1200", "1680x1050", "320x200"};
 static const int nbRes = sizeof(Res) / sizeof(Res[0]);
 #endif // USE_RANDR_EXT
 
@@ -196,9 +196,10 @@ gfScreenInit(void)
 						}
 					}
 				}
+
+				nbRes = nsize + add_modes;
 			}
 
-			nbRes = nsize;
 			XRRFreeScreenConfigInfo(screenconfig);
 		}
 		XCloseDisplay(display);
