@@ -4,7 +4,7 @@
     created              : Mon Aug 21 20:55:32 CEST 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id$
+    version              : $Id: grcam.cpp,v 1.49 2006/02/22 17:53:06 olethros Exp $
 
  ***************************************************************************/
 
@@ -111,7 +111,15 @@ cGrPerspCamera::cGrPerspCamera(class cGrScreen *myscreen, int id, int drawCurr,
 
 void cGrPerspCamera::setProjection(void)
 {
-    grContext.setFOV(screen->getViewRatio() * fovy, fovy);
+    // PLib takes the field of view as angles in degrees. However, the
+    // aspect ratio really aplies to lengths in the projection
+    // plane. So we have to transform the fovy angle to a length in
+    // the projection plane, apply the aspect ratio and transform the
+    // result back to an angle. Care needs to be taken to because the
+    // tan and atan functions operate on angles in radians. Also,
+    // we're only interested in half the viewing angle.
+    float fovx = atan(screen->getViewRatio() * tan(fovy * M_PI / 360.0)) * 360.0 / M_PI;
+    grContext.setFOV(fovx, fovy);
     grContext.setNearFar(fnear, ffar);
 }
 
@@ -214,6 +222,7 @@ void cGrOrthoCamera::setModelView(void)
 
 void cGrBackgroundCam::update(cGrCamera *curCam)
 {
+    static float BACKGROUND_FOVY_CUTOFF = 60.0f;
     memcpy(&eye, curCam->getPosv(), sizeof(eye));
     memcpy(&center, curCam->getCenterv(), sizeof(center));
     sgSubVec3(center, center, eye);
@@ -221,6 +230,10 @@ void cGrBackgroundCam::update(cGrCamera *curCam)
     speed[0]=0.0;
     speed[1]=0.0;
     speed[2]=0.0;
+    fovy = curCam->getFovY();
+    if (fovy < BACKGROUND_FOVY_CUTOFF) {
+        fovy = BACKGROUND_FOVY_CUTOFF;
+    }
     memcpy(&up, curCam->getUpv(), sizeof(up));
 }
 
@@ -241,12 +254,6 @@ class cGrCarCamInside : public cGrPerspCamera
 	up[2] = 1;
     }
 
-    void limitFov(void) {
-	if ((screen->getViewRatio() * fovy) > 90.0) {
-	    fovy = 90.0 / screen->getViewRatio();
-	}
-    }
-
     void update(tCarElt *car, tSituation *s) {
 	sgVec3 P, p;
 	
@@ -421,12 +428,6 @@ class cGrCarCamInsideFixedCar : public cGrPerspCamera
 			 myfnear, myffar, myfogstart, myfogend) {
     }
 
-    void limitFov(void) {
-	if ((screen->getViewRatio() * fovy) > 90.0) {
-	    fovy = 90.0 / screen->getViewRatio();
-	}
-    }
-
     void update(tCarElt *car, tSituation *s) {
 	sgVec3 P, p;
 	
@@ -482,12 +483,6 @@ class cGrCarCamBehind : public cGrPerspCamera
 	up[2] = 1;
     }
 
-    void limitFov(void) {
-	if ((screen->getViewRatio() * fovy) > 90.0) {
-	    fovy = 90.0 / screen->getViewRatio();
-	}
-    }
-
     void update(tCarElt *car, tSituation *s) {
 	tdble A;
 	tdble CosA;
@@ -543,12 +538,6 @@ class cGrCarCamBehind2 : public cGrPerspCamera
 	up[2] = 1;
     }
 
-    void limitFov(void) {
-	if ((screen->getViewRatio() * fovy) > 90.0) {
-	    fovy = 90.0 / screen->getViewRatio();
-	}
-    }
-
     void update(tCarElt *car, tSituation *s) {
 	tdble A;
 	tdble CosA;
@@ -602,12 +591,6 @@ class cGrCarCamFront : public cGrPerspCamera
 	up[2] = 1;
     }
 
-    void limitFov(void) {
-	if ((screen->getViewRatio() * fovy) > 90.0) {
-	    fovy = 90.0 / screen->getViewRatio();
-	}
-    }
-
     void update(tCarElt *car, tSituation *s) {
 	tdble CosA = cos(car->_yaw);
 	tdble SinA = sin(car->_yaw);
@@ -653,12 +636,6 @@ protected:
 	up[2] = 1;
     }
 
-    void limitFov(void) {
-	if ((screen->getViewRatio() * fovy) > 90.0) {
-	    fovy = 90.0 / screen->getViewRatio();
-	}
-    }
-
     void update(tCarElt *car, tSituation *s) {
 	tdble x = car->_pos_X + distx;
 	tdble y = car->_pos_Y + disty;
@@ -717,12 +694,6 @@ class cGrCarCamUp : public cGrPerspCamera
 	}
     }
 
-    void limitFov(void) {
-	if ((screen->getViewRatio() * fovy) > 90.0) {
-	    fovy = 90.0 / screen->getViewRatio();
-	}
-    }
-
     void update(tCarElt *car, tSituation *s) {
 	tdble x = car->_pos_X;
 	tdble y = car->_pos_Y;
@@ -771,12 +742,6 @@ class cGrCarCamCenter : public cGrPerspCamera
 	up[2] = 1;
     }
 
-    void limitFov(void) {
-	if ((screen->getViewRatio() * fovy) > 90.0) {
-	    fovy = 90.0 / screen->getViewRatio();
-	}
-    }
-
     void loadDefaults(char *attr) {
 	sprintf(path, "%s/%d", GR_SCT_DISPMODE, screen->getId());
 	locfovy = (float)GfParmGetNum(grHandle, path,
@@ -879,12 +844,6 @@ class cGrCarCamLookAt : public cGrPerspCamera
 	}
     }
 
-    void limitFov(void) {
-	if ((screen->getViewRatio() * fovy) > 90.0) {
-	    fovy = 90.0 / screen->getViewRatio();
-	}
-    }
-
     void update(tCarElt *car, tSituation *s) {
     }
 };
@@ -906,15 +865,10 @@ class cGrCarCamRoadNoZoom : public cGrPerspCamera
 	up[2] = 1;
     }
 
-    void limitFov(void) {
-	if ((screen->getViewRatio() * fovy) > 90.0) {
-	    fovy = 90.0 / screen->getViewRatio();
-	}
-    }
-
     void update(tCarElt *car, tSituation *s) {
 	tRoadCam *curCam;
 
+
 	curCam = car->_trkPos.seg->cam;
     
 	if (curCam == NULL) {
@@ -981,62 +935,79 @@ class cGrCarCamRoadFly : public cGrPerspCamera
 	    currenttime = s->currentTime;
 	}
 	
-	if (currenttime != s->currentTime) {
-	    dt = s->currentTime - currenttime;
-	    currenttime = s->currentTime;
+	if (currenttime == s->currentTime) {
+            return;
+        }
 
-	    timer--;
-	    if (timer<0) {
-		eye[0] = car->_pos_X + 50.0 + (50.0*rand()/(RAND_MAX+1.0));
-		eye[1] = car->_pos_Y + 50.0 + (50.0*rand()/(RAND_MAX+1.0));
-		eye[2] = car->_pos_Z + 50.0 + (50.0*rand()/(RAND_MAX+1.0));
-	    }
+        bool reset_camera = false;
+        dt = s->currentTime - currenttime;
+        currenttime = s->currentTime;
+        if (fabs(dt) > 1.0f) { 
+            dt = 0.1f; // avoid overflow
+            reset_camera = true;
+        }
 
-	    if (current != car->index) {
-		/* the target car changed */
-		zOffset = 50.0;
-		current = car->index;
-	    } else {
-		zOffset = 0.0;
-	    }
+        timer--;
+        if (timer<0) {
+            reset_camera = true;
+        }
 
-	    if ((timer <= 0) || (zOffset > 0.0)) {
-		timer = 500 + (int)(500.0*rand()/(RAND_MAX+1.0));
-		offset[0] = -0.5 + (rand()/(RAND_MAX+1.0));
-		offset[1] = -0.5 + (rand()/(RAND_MAX+1.0));
-		offset[2] = 30 + (30.0*rand()/(RAND_MAX+1.0)) + zOffset;
-		offset[0] = offset[0]*(offset[2]+1.0);
-		offset[1] = offset[1]*(offset[2]+1.0);
-		gain = 5.0;
-		damp = 5.0;
-	    }
+        if (current != car->index) {
+            /* the target car changed */
+            zOffset = 50.0;
+            current = car->index;
+            reset_camera = true;
+        } else {
+            zOffset = 0.0;
+        }
+
+        if ((timer <= 0) || (zOffset > 0.0)) {
+            timer = 500 + (int)(500.0*rand()/(RAND_MAX+1.0));
+            offset[0] = -0.5 + (rand()/(RAND_MAX+1.0));
+            offset[1] = -0.5 + (rand()/(RAND_MAX+1.0));
+            offset[2] = 10.0f + (50.0*rand()/(RAND_MAX+1.0)) + zOffset;
+            offset[0] = offset[0]*(offset[2]+1.0);
+            offset[1] = offset[1]*(offset[2]+1.0);
+            // follow the car more closely when low
+            gain = 300.0/(10.0f+offset[2]); 
+            damp = 5.0f;
+        }
+
+
+        if (reset_camera) {
+            eye[0] = car->_pos_X + 50.0 + (50.0*rand()/(RAND_MAX+1.0));
+            eye[1] = car->_pos_Y + 50.0 + (50.0*rand()/(RAND_MAX+1.0));
+            eye[2] = car->_pos_Z + 50.0 + (50.0*rand()/(RAND_MAX+1.0));
+            speed[0] = speed[1] = speed[2] = 0.0f;
+        }
 	
-	    speed[0] += (gain*(offset[0]+car->_pos_X - eye[0]) - speed[0]*damp)*dt;
-	    speed[1] += (gain*(offset[1]+car->_pos_Y - eye[1]) - speed[1]*damp)*dt;
-	    speed[2] += (gain*(offset[2]+car->_pos_Z - eye[2]) - speed[2]*damp)*dt;
+        speed[0] += (gain*(offset[0]+car->_pos_X - eye[0]) - speed[0]*damp)*dt;
+        speed[1] += (gain*(offset[1]+car->_pos_Y - eye[1]) - speed[1]*damp)*dt;
+        speed[2] += (gain*(offset[2]+car->_pos_Z - eye[2]) - speed[2]*damp)*dt;
 
-	    eye[0] = eye[0] + speed[0]*dt;
-	    eye[1] = eye[1] + speed[1]*dt;
-	    eye[2] = eye[2] + speed[2]*dt;
+        eye[0] = eye[0] + speed[0]*dt;
+        eye[1] = eye[1] + speed[1]*dt;
+        eye[2] = eye[2] + speed[2]*dt;
 
-	    center[0] = (car->_pos_X);
-	    center[1] = (car->_pos_Y);
-	    center[2] = (car->_pos_Z);
+        center[0] = (car->_pos_X);
+        center[1] = (car->_pos_Y);
+        center[2] = (car->_pos_Z);
 
-	    // avoid going under the scene
-	    height = grGetHOT(eye[0], eye[1]) + 1.0;
-	    if (eye[2] < height) {
-		timer = 500 + (int)(500.0*rand()/(RAND_MAX+1.0));
-		offset[2] = height - car->_pos_Z + 1.0;
-		eye[2] = height;
-	    }
+        // avoid going under the scene
+        height = grGetHOT(eye[0], eye[1]) + 1.0;
+        if (eye[2] < height) {
+            timer = 500 + (int)(500.0*rand()/(RAND_MAX+1.0));
+            offset[2] = height - car->_pos_Z + 1.0;
+            eye[2] = height;
+        }
 	
-	}
-
     }
 
+
+
     void onSelect(tCarElt *car, tSituation *s)
     {
+        printf ("%f select\n", s->currentTime);
 	timer = 0;
 	current = -1;
     }
@@ -1064,12 +1035,6 @@ class cGrCarCamRoadZoom : public cGrPerspCamera
 	up[2] = 1;
     }
 
-    void limitFov(void) {
-	if ((screen->getViewRatio() * fovy) > 90.0) {
-	    fovy = 90.0 / screen->getViewRatio();
-	}
-    }
-
     void loadDefaults(char *attr) {
 	sprintf(path, "%s/%d", GR_SCT_DISPMODE, screen->getId());
 	locfovy = (float)GfParmGetNum(grHandle, path,
@@ -1092,7 +1057,6 @@ class cGrCarCamRoadZoom : public cGrPerspCamera
 	    eye[0] = grWrldX * 0.5;
 	    eye[1] = grWrldY * 0.6;
 	    eye[2] = 120;
-
 	} else {
 	    eye[0] = curCam->pos.x;
 	    eye[1] = curCam->pos.y;
