@@ -4,7 +4,7 @@
     created              : Sun Mar 19 00:07:42 CET 2000
     copyright            : (C) 2000 by Eric Espie
     email                : torcs@free.fr
-    version              : $Id: sim.h,v 1.11 2004/04/06 03:00:43 torcs Exp $
+    version              : $Id: sim.h,v 1.15 2005/07/19 19:55:55 berniw Exp $
 
  ***************************************************************************/
 
@@ -39,7 +39,7 @@
 extern void SimConfig(tCarElt *carElt, RmInfo *);
 extern void SimReConfig(tCarElt *carElt);
 extern void SimUpdate(tSituation*, double deltaTime, int telemetry);
-extern void SimInit(int nbcars);
+extern void SimInit(int nbcars, tTrack* track);
 extern void SimShutdown(void);
 
 extern void SimAxleConfig(tCar *car, int index);
@@ -71,7 +71,7 @@ extern void SimBrakeSystemUpdate(tCar *car);
 extern void SimAeroConfig(tCar *car);
 extern void SimAeroUpdate(tCar *car, tSituation *s);
 extern void SimWingConfig(tCar *car, int index);
-extern void SimWingUpdate(tCar *car, int index);
+extern void SimWingUpdate(tCar *car, int index, tSituation *s);
 
 extern void SimCarUpdateWheelPos(tCar *car);
 
@@ -90,10 +90,10 @@ extern void SimEngineShutdown(tCar *car);
 extern void SimCarCollideZ(tCar *car);
 extern void SimCarCollideXYScene(tCar *car);
 extern void SimCarCollideCars(tSituation*);
-extern void SimCarCollideConfig(tCar *car);
+extern void SimCarCollideConfig(tCar *car, tTrack *track);
 extern void SimCarCollideShutdown(int nbcars);
 extern void SimCarCollideInit(void);
-
+extern void SimCollideRemoveCar(tCar *car, int nbcars);
 
 extern tdble SimDeltaTime;
 extern int SimTelemetry;
@@ -104,6 +104,11 @@ extern t3Dd vectEnd[];
 extern tdble simDammageFactor[];
 extern tdble simSkidFactor[];
 
+/// return a number drawn uniformly from [0,1]
+inline float urandom() {
+	return ((((float)rand()-1)/((float)RAND_MAX)));
+}
+
 #define SIM_VECT_COLL	12
 #define SIM_VECT_SPD	13
 #define SIM_VECT_ACCEL	14
