@@ -4,7 +4,7 @@
     created      : Sun Dec 24 12:14:18 CET 2000
     copyright    : (C) 2000 by Eric Espie
     email        : eric.espie@torcs.org
-    version      : $Id: track.cpp,v 1.21 2005/07/19 19:55:56 berniw Exp $
+    version      : $Id: track.cpp,v 1.22 2006/10/29 15:55:33 berniw Exp $
 
  ***************************************************************************/
 
@@ -20,7 +20,7 @@
 /** @file   
     		
     @author	<a href=mailto:eric.espie@torcs.org>Eric Espie</a>
-    @version	$Id: track.cpp,v 1.21 2005/07/19 19:55:56 berniw Exp $
+    @version	$Id: track.cpp,v 1.22 2006/10/29 15:55:33 berniw Exp $
 */
 
 #include <stdio.h>
@@ -392,7 +392,8 @@ InitScene(tTrack *Track, void *TrackHandle, int bump)
 	    trackvertices[3*nbvert]   = x;			\
 	    trackvertices[3*nbvert+1] = y;			\
 	    trackvertices[3*nbvert+2] = z;			\
-	    trackindices[nbvert]      = nbvert++;		\
+	    trackindices[nbvert]      = nbvert;		\
+	    ++nbvert;								\
 	}							\
     } while (0)
 
@@ -403,7 +404,8 @@ InitScene(tTrack *Track, void *TrackHandle, int bump)
 	    trackvertices[3*nbvert]   = x;			\
 	    trackvertices[3*nbvert+1] = y;			\
 	    trackvertices[3*nbvert+2] = z;			\
-	    trackindices[nbvert]      = nbvert++;		\
+	    trackindices[nbvert]      = nbvert;		\
+	    ++nbvert;								\
 	    printf("x=%f y=%f z=%f  u=%f v=%f\n",		\
 		   (tdble)(x), (tdble)(y), (tdble)(z),		\
 		   (tdble)(t1), (tdble)(t2));			\
